<?php
/**
 * Plugin Helper File
 *
 * @package     Tabber
 * @version     0.1.2a
 *
 * @author      Peter van Westen <peter@nonumber.nl>
 * @link        http://www.nonumber.nl
 * @copyright   Copyright (C) 2010 NoNumber! All Rights Reserved
 * @license     http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
 */

// No direct access
defined( '_JEXEC' ) or die( 'Restricted access' );

/**
* Plugin that replaces stuff
*/
class plgSystemTabberHelper
{
	function plgSystemTabberHelper( &$config )
	{
		// Load plugin parameters
		require_once JPATH_PLUGINS.DS.'system'.DS.'nonumberelements'.DS.'helpers'.DS.'parameters.php';
		$parameters =& NNePparameters::getParameters();
		$this->params = $parameters->getParams( $config['params'], JPATH_PLUGINS.DS.$config['type'].DS.$config['name'].'.xml' );

		$this->params->hastabs = 0;

		$this->params->comment_start = '<!-- START: Tabber -->';
		$this->params->comment_end = '<!-- END: Tabber -->';

		$break_tags_start =	'((?:<p(?: [^>]*)?>\s*)?)';
		$break_tags_end =	'((?:\s*</p>)?)';

		$this->params->tag_open =		preg_replace( '#[^a-z0-9-_]#s', '', $this->params->tag_open );
		$this->params->tag_close =		preg_replace( '#[^a-z0-9-_]#s', '', $this->params->tag_close );
		$this->params->tag_tablink =	preg_replace( '#[^a-z0-9-_]#s', '', $this->params->tag_tablink );
		$this->params->tag_delimiter =	( $this->params->tag_delimiter == 'space' ) ? ' ' : '=';

		$this->params->regex =			'#'.$break_tags_start.'\{('.preg_quote( $this->params->tag_open, '#' ).preg_quote( $this->params->tag_delimiter, '#' ).'([^\}]*)|/'.preg_quote( $this->params->tag_close, '#' ).')\}'.$break_tags_end.'#s';
		$this->params->regex_end =		'#'.$break_tags_start.'\{/'.preg_quote( $this->params->tag_close, '#' ).'\}'.$break_tags_end.'#s';
		$this->params->regex_link =		'#\{'.preg_quote( $this->params->tag_tablink, '#' ).preg_quote( $this->params->tag_delimiter, '#' ).'([^\}]*)\}(.*?)\{/'.preg_quote( $this->params->tag_tablink, '#' ).'\}#s';

		$this->params->protect_s = array( '{'.$this->params->tag_open, '{/'.$this->params->tag_close, '{'.$this->params->tag_tablink );
		$this->params->protect_r = array( $this->protectStr( $this->params->protect_s['0'] ), $this->protectStr( $this->params->protect_s['1'] ), $this->protectStr( $this->params->protect_s['2'] ) ) ;
	}


////////////////////////////////////////////////////////////////////
// onAfterDispatch
////////////////////////////////////////////////////////////////////
	function onAfterDispatch()
	{
		$document =& JFactory::getDocument();
		$docType = $document->getType();

		// only in html
		if ( $docType !== 'html' ) { return; }

		JHTML::_('behavior.mootools');

		require_once JPATH_PLUGINS.DS.'system'.DS.'nonumberelements'.DS.'helpers'.DS.'functions.php';
		$this->functions =& NNFunctions::getFunctions();
		$mt_version = $this->functions->getJSVersion();

		$script =
			 'var tabber_slide_speed = '.(int) $this->params->tabber_slide_speed.';'
			.'var tabber_fade_in_speed = '.(int) $this->params->tabber_fade_in_speed.';'
			.'var tabber_scroll = '.(int) $this->params->tabber_scroll.';';
		$document->addScriptDeclaration( $script );
		$document->addScript( JURI::root( true ).'/plugins/system/tabber/js/script'.$mt_version.'.js' );
		$document->addStyleSheet( JURI::root( true ).'/plugins/system/tabber/css/style.css' );

		$buffer = $document->getBuffer( 'component' );

		if ( !$buffer || strpos( $buffer, '{'.$this->params->tag_open ) === false ) { return; }

		$this->params->hastabs = 1;

		$this->protect( $buffer );
		$this->replaceTags( $buffer );
		$this->unprotect( $buffer );

		$document->setBuffer( $buffer, 'component' );
	}

////////////////////////////////////////////////////////////////////
// onAfterRender
////////////////////////////////////////////////////////////////////
	function onAfterRender()
	{
		$document =& JFactory::getDocument();
		$docType = $document->getType();

		// only in html
		if ( $docType !== 'html' ) { return; }

		$html = JResponse::getBody();
		if ( $html == '' ) { return; }
				
		if ( strpos( $html, '{'.$this->params->tag_open ) === false ) {
			if ( !$this->params->hastabs ) {
				// remove style and script if no tabs are found
				$html = preg_replace( '#\s*<link rel="stylesheet" href="[^"]*/plugins/system/tabber/css/style.css" type="text/css" />#s', '', $html );
				$html = preg_replace( '#\s*<script type="text/javascript" src="[^"]*/plugins/system/tabber/js/script[^"]*.js"></script>#s', '', $html );
			}
		} else {
			if ( !( strpos( $html, '<body' ) === false ) && !( strpos( $html, '</body>' ) === false ) ) {
				$html_split = explode( '<body', $html );
				$body_split = explode( '</body>', $html_split['1'] );

				// only do the handling inside the body
				$this->protect( $body_split['0'] );
				$this->replaceTags( $body_split['0'] );
				$this->unprotect( $body_split['0'] );

				$html_split['1'] = implode( '</body>', $body_split );
				$html = implode( '<body', $html_split );
			} else {
				$this->protect( $html );
				$this->replaceTags( $html );
				$this->unprotect( $html );
			}
		}

		JResponse::setBody( $html );
	}

////////////////////////////////////////////////////////////////////
// FUNCTIONS
////////////////////////////////////////////////////////////////////
	function replaceTags( &$str )
	{
		$alltabs = array();
		$sets = array();
		if ( preg_match_all( $this->params->regex, $str, $matches, PREG_SET_ORDER ) > 0 ) {
			$t = 0;
			$set = 1;
			foreach ( $matches as $match ) {
				if ( $match['2'] == '/'.$this->params->tag_close ) {
					$set++;
					continue;
				}
				$tab = '';
				$tab->orig = $match['0'];
				$tab->title = $match['3'];
				$tab->set = $set;
				$tab->pre = '';
				$tab->post = '';
				if ( !$match['1'] || !$match['4'] ) {
					$tab->pre = trim( $match['4'] );
					$tab->post = trim( $match['1'] );
				}
				if ( !isset( $sets[$set] ) ) {
					$sets[$set] = array();
				}
				$sets[$set][] = $tab;
			}
		}

		$urltab = strtolower( JRequest::getString( 'tab' ) );

		foreach ( $sets as $set_id => $tabs ) {
			$rand = rand( 100, 999 );
			$active_by_url = 0;
			$active = 0;
			foreach ( $tabs as $i => $tab ) {
				$tab->class = '';
				$tab->active = 0;
				if ( !( strpos( $tab->title, '|' ) === false ) ) {
					list( $tab->title, $extra ) = explode( '|', $tab->title, 2 );
					$extra = explode( '|', $extra );
					foreach ( $extra as $e ) {
						switch ( $e ) {
							case 'active':
								$active = $i;
								break;
							default:
								$tab->class = trim( $tab->class.' '.$e );
								break;
						}
					}
				}

				$tab->id = 'tab_'.$set_id.'-'.($i+1).'_'.$rand;

				$tab->matches = array();
				$tab->matches[] = $tab->title;
				$tab->matches[] = str_replace( ' ', '', $tab->title );
				$tab->matches[] = strtolower( $tab->title );
				$tab->matches[] = str_replace( ' ', '', strtolower( $tab->title ) );
				$alias = trim( preg_replace( '#[^a-z0-9]#s', '', strtolower( $tab->title ) ) );
				if ( $alias ) {
					$tab->matches[] = $alias;
				}
				$tab->matches[] = $i+1;
				$tab->matches[] = $set_id.'-'.($i+1);

				if ( $urltab != '' && ( in_array( $urltab, $tab->matches, 1 ) || in_array( strtolower( $urltab ), $tab->matches, 1 ) ) ) {
					$active_by_url =  $i;
				}

				$sets[$set_id][$i] = $tab;
				$alltabs[] = $tab;
			}
			if ( $active_by_url ) {
				foreach( $tabs as $i => $t ) {
					$sets[$set_id][$i]->active = ( $active_by_url == $i );
				}
			} else if ( isset( $sets[$set_id][$active] ) ) {
				$sets[$set_id][$active]->active = 1;
			}
		}

		foreach ( $sets as $tabs ) {
			$rand = rand( 100, 999 );
			foreach ( $tabs as $i => $tab ) {
				$html = array();
				if ( $i == 0 ) {
					$html[] = '<div class="'.trim( 'tabber_container '.( $this->params->rounded ? 'rounded' : '' ) ).'" id="tabber_container_'.$i.'_'.$rand.'">';
					$html[] = $this->getNav( $tabs );
					$html[] = '<div class="tabber_content">';
				} else {
					$html[] = '<div style="clear:both;"></div>';
					$html[] = '</div>';
				}

				$html[] = '<div class="'.trim( 'tabber_item '.trim( $tab->class ) ).'" id="item_'.$tab->id.'">';
				$html[] = '<a name="'.$tab->id.'"></a><h2 class="tabber_title">'.$tab->title.'</h2>';

				$s = '#'.preg_quote( $tab->orig, '#' ).'#';
				if ( @preg_match( $s.'u', $str ) ) {
					$s .= 'u';
				}
				if ( preg_match( $s, $str ) ) {
					$str = preg_replace( $s, implode( "\n", $html ), $str, 1 );
				}
			}
		}

		// closing html
		$html = array();
		$html[] = '<div style="clear:both;"></div>';
		$html[] = '</div>';
		$html[] = '<div style="clear:both;"></div>';
		$html[] = '</div>';
		$html[] = '</div>';
		$html = implode( "\n", $html );

		if ( preg_match_all( $this->params->regex_end, $str, $matches, PREG_SET_ORDER ) > 0 ) {
			foreach ( $matches as $match ) {
				$m_html = $html;
				if ( !$match['1'] || !$match['2'] ) {
					$m_html = trim( $match['2'] ).$m_html.trim( $match['1'] );
				}
				$str = str_replace( $match['0'], $m_html, $str );
			}
		}
		
		if ( preg_match_all( $this->params->regex_link, $str, $matches, PREG_SET_ORDER ) > 0 ) {
			foreach ( $matches as $match ) {
				$link = $match['2'];
				$tab = 0;
				$name = $match['1'];
				foreach ( $alltabs as $t ) {
					if ( in_array( $name, $t->matches, 1 ) || in_array( strtolower( $name ), $t->matches, 1 ) ) {
						$tab = $t;
						break;
					}
				}
				if ( $tab ) {
					$link = '<a href="#'.$tab->id.'" class="tabber_tablink" rel="'.$tab->id.'">'.$link.'</a>';
				}
				$str = str_replace( $match['0'], $link, $str );
			}
		}
	}

	function getNav( &$tabs )
	{
		$html[] = '<div class="tabber_nav" style="display:none;">';
			$html[] = '<ul class="tabber_tabs">';
			foreach( $tabs as $tab ) {
				$html[] = '<li class="'.trim( 'tabber_tab'.( $tab->active ? ' active' : '' ).' '.trim( $tab->class ) ).'" id="'.$tab->id.'"><a href="javascript://" onfocus="this.blur()"><span>'.$tab->title.'</span></a></li>';
			}
			$html[] = '</ul>';
		$html[] = '</div>';

		return implode( "\n", $html );
	}

	/*
	 * Protect admin form
	 */
	function protect( &$str )
	{
		$option = JRequest::getCmd( 'option' );
		if (	JRequest::getCmd( 'task' ) == 'edit'
			||	JRequest::getCmd( 'layout' ) == 'form'
			||	JRequest::getCmd( 'layout' ) == 'write'
			||	$option == 'com_contentsubmit'
		) {
			// Protect complete adminForm (to prevent articles from being created when editing articles and such)
			$regex = '#<form [^>]*(id|name)="adminForm".*?</form>#si';
			if ( preg_match_all( $regex, $str, $matches, PREG_SET_ORDER ) > 0 ) {
				foreach ( $matches as $match ) {
					$str = str_replace( $match[0], str_replace( $this->params->protect_s, $this->params->protect_r, $match[0] ), $str );
				}
			}
		}
	}

	function unprotect( &$str )
	{
		$str = str_replace( $this->params->protect_r, $this->params->protect_s, $str );
	}

	function protectStr( $str )
	{
		return base64_encode( $str );
	}
}