/**
 * Main JavaScript file (MooTools 1.2 compatible)
 *
 * @package     Tabber
 * @version     1.0.2
 *
 * @author      Peter van Westen <peter@nonumber.nl>
 * @link        http://www.nonumber.nl
 * @copyright   Copyright (C) 2010 NoNumber! All Rights Reserved
 * @license     http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
 */

window.addEvent( 'domready', function() {
	// Only do stuff if tabber_nav is found
	if ( document.getElements( 'div.tabber_nav' ).length ) {
		Tabber = new Tabber();
	} else {
		// Try again 2 seconds later, because IE sometimes can't see object immediatly
		(function() {
			if ( document.getElements( 'div.tabber_nav' ).length ) {
				Tabber = new Tabber();
			}
		}).delay( 2000 );
	}
});

var Tabber = new Class({
	initialize: function()
	{
		var self = this
		this.docScroll = new Fx.Scroll( window );
		this.containers = new Array();

		document.getElements( 'div.tabber_container' ).each( function( container ) {
			var active = 0;

			container.getElements( 'div.tabber_content' ).each( function( el ) {
				el.fx = new Fx.Tween( el, { 'duration' : tabber_slide_speed, onComplete: function() { self.autoHeight( el ); } } );
			});

			// add onclick events on tabs
			container.getElements( 'li.tabber_tab' ).each( function( el, i ) {
				self.containers[el.id] = container.id;

				// set first tab as active or active tab
				if ( ( !i && !el.hasClass( 'inactive' ) ) || el.hasClass( 'active' ) ) {
					active = el.id;
				}
				el.addEvent( 'click', function() { self.showTab( el.id, container.id ); } );
			});

			// add fx
			container.getElements( 'div.tabber_item' ).each( function( el ) {
				el.fade_in = new Fx.Styles( el, { 'duration' : tabber_fade_in_speed } );
				el.fx = new Fx.Slide( el, { 'duration' : 0, onComplete: function() { self.autoHeight( el.getParent() ); } } );
				el.fx.hide();
			});

			// show tabs list
			container.getElements( 'div.tabber_nav' ).each( function( el ) {
				el.setStyle( 'display', 'block' );
			});

			// hide content titles
			container.getElements( 'h2.tabber_title' ).each( function( el ) {
				el.setStyle( 'display', 'none' );
			});

			// show only active tab
			self.showTab( active, container.id, 1 );
		});

		// add onclick events on tab links {tablink=...}
		document.getElements( 'a.tabber_tablink' ).each( function( el ) {
			if ( el.rel && typeof self.containers[el.rel] !== 'undefined' ) {
				el.addEvent( 'click', function() {
					self.showTab( el.rel, self.containers[el.rel], 0, 1 );
				} );
				el.href = 'javascript://';
			}
		});
	},

	showTab: function( id, c_id, first, scroll )
	{
		var self = this;
		var container = document.id( c_id );
		var item = document.id( id );
		var isactive = ( item && item.hasClass( 'active' ) );

		// remove all active classes
		container.getElements( 'li.tabber_tab' ).each( function( el ) {
			if ( el.hasClass( 'active' ) ) {
				el.removeClass( 'active' );
				content = $( 'item_'+el.id );
				if ( content ) {
					content.getParent().getParent().setStyle( 'height', parseInt( content.getStyle( 'height' ) ) );
				}
			}
		});

		if ( item ) {
			item.addClass( 'active' );
		}

		var el = document.id( 'item_'+id );

		// show active content block
		if ( el ) {
			var content = el.getParent().getParent();
			el.fx.cancel();
			if ( first ) {
				el.fx.show();
				this.autoHeight( content );
				this.autoHeight( el.getParent() );
			} else if ( !isactive ) {
				// show active content block
				el.fade_in.cancel();
				el.setStyle( 'opacity', 0 );
				el.fx.show();
				this.autoHeight( el.getParent() );
				el.fade_in.start( { 'opacity' : 1 } );
				content.fx.cancel().start( 'height', parseInt( el.getStyle( 'height' ) ) );
			}
			if ( ( tabber_scroll && !first ) || scroll ) {
				this.docScroll.cancel().toElement( container );
			}
		}

		// hide all content block
		container.getElements( 'div.tabber_item' ).each( function( el ) {
			if ( id && el.id && el.id != 'item_'+id ) {
				el.fx.hide();
			}
		});
	},

	autoHeight: function( el )
	{
		if ( el.getStyle( 'height' ) && parseInt( el.getStyle( 'height' ) ) > 0 ) {
			el.setStyle( 'height', 'auto' );
		}
	}
});
